/*
 * $Id: error_msg.c,v 1.4 1995/11/28 19:12:28 jik Exp $
 * $Locker:  $
 *
 * Copyright 1987 by the Student Information Processing Board
 * of the Massachusetts Institute of Technology
 *
 * For copyright info, see "sipb-copying.h".
 */

#include <stdio.h>
#include "error_table.h"
#include "sipb-copying.h"
#include "internal.h"

static const char rcsid[] =
    "$Id";
static const char copyright[] =
    "Copyright 1986, 1987, 1988 by the Student Information Processing Board\nand the department of Information Systems\nof the Massachusetts Institute of Technology";

static char buffer[25];

struct et_list * _et_list = (struct et_list *) NULL;

const char * error_message (code)
long	code;
{
    int offset;
    struct et_list *et;
    int table_num;
    int started = 0;
    char *cp;

    offset = code & ((1<<ERRCODE_RANGE)-1);
    table_num = code - offset;
    if (!table_num) {
#ifdef POSIX
      /*
	XXX At least on Linux, strerror() is defined as returning a
	string that is only valid until the next time strerror() is
	called.  So what happens if the programmer tries to use
	error_message() to get multiple errno error strings?  This is
	unlikely enough that I'm not going to worry about it.
	*/
      return strerror(offset);
#else
	if (offset < sys_nerr)
	    return(sys_errlist[offset]);
	else
	    goto oops;
#endif
    }
    for (et = _et_list; et; et = et->next) {
	if (et->table->base == table_num) {
	    /* This is the right table */
	    if (et->table->n_msgs <= offset)
		goto oops;
	    return(et->table->msgs[offset]);
	}
    }
oops:
    strcpy (buffer, "Unknown code ");
    if (table_num) {
	strcat (buffer, error_table_name (table_num));
	strcat (buffer, " ");
    }
    for (cp = buffer; *cp; cp++)
	;
    if (offset >= 100) {
	*cp++ = '0' + offset / 100;
	offset %= 100;
	started++;
    }
    if (started || offset >= 10) {
	*cp++ = '0' + offset / 10;
	offset %= 10;
    }
    *cp++ = '0' + offset;
    *cp = '\0';
    return(buffer);
}
